/****************************************************************************
**
** Copyright (C) 2010  Aimone Davide realsportcars@hotmail.com
**
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation, either version 3 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
****************************************************************************/

#ifndef TEXTPROCESSOR_H
#define TEXTPROCESSOR_H

#include <QWidget>
#include <QTextEdit>
#include <QDesktopServices>
#include <QGridLayout>
#include "recipient.h"
#include "core.h"
#include "dictionary.h"

namespace Ui {
    class TextProcessor;
}

class TextProcessor : public QWidget
{
    Q_OBJECT

public:
    explicit TextProcessor(QWidget *parent = 0);
    void setDictionary(dictionary *dict);
    void setSmsFontSize(int nSize);
    ~TextProcessor();
    QList<QString> m_lWord;
    Recipient *m_recipient;
    void enableNewWordLayout();
    void addNewWord(QString *sWord);
    void setText(QString sWord);
    QString getText();
    //Test
    QPushButton *getSendBt();
    void setStandalone(bool bStandalone = true);


private:
    dictionary *m_dict;
    Ui::TextProcessor *ui;
    QTextEdit *m_smsEdit;
    QLineEdit *m_phoneNumber;
    void addText(QString sWord);
    void processQwerty(QString sInput);
    void processAlfanumeric(QString sInput);
    void resetFastWriteStatus();
    void loadDict();
    bool m_bFastInput;
    int m_nInputSyle;
    int m_nState;
    int m_nMsgLength;
    QString m_sCompleteValue;
    int m_nWordIndex;
    int m_nBeginWord;
    int m_nEndWord;
    QTextCursor m_cursorSmsText;
    bool m_bTimerNewChar;
    QString m_sLastPress;
    bool m_bRememberUppercase;
    void updateCharCounter(int action);
    void updateText(QString sWord);
    QTimer *m_timerNewChar;
    QString capitalizeIfNeeded(QString sWord);
    int m_nCharCount;
    int m_nSMSCount;
    QString m_sCapitalization;
    void processDigit(QString sInput);
    void moveCursorBt(bool bMoveRight);
    void deleteChar();
    void nextWord();
    void prevWord();
    QString *m_sMessage;
    int m_nSavedStyle;
    bool m_bQwertyEnable;
    bool m_bStandalone;

public slots:
    void processInput(QString, Core::Keyboards);
    void settingsChanged(int);
    void sendSms();
    void commit();

private slots:
    void alarmNewChar();
    void cursorPosChanged();
    void smsChanged();

};

#endif // TEXTPROCESSOR_H
